package com.muyu.common.domain.enums;

import java.math.BigDecimal;
import java.util.Date;

/**
 * @Author：李庆帅
 * @Package：com.muyu.etl.property.enums
 * @Project：cloud-etl-property
 * @name：DataType
 * @Date：2024/8/29 21:33
 *
 * 定义一个枚举，用于映射数据库类型到Java类型
 */
public enum DataType {
    // 定义各种数据库类型到Java类型的映射
    VARCHAR("varchar",String.class,"String"),
    BIGINT("bigint", Long.class,"Long"),
    INT("int", Integer.class,"Integer"),
    DECIMAL("decimal", BigDecimal.class,"BigDecimal"),
    DATETIME("datetime", Date.class,"Date"),
    TEXT("text", String.class,"String"),
    DOUBLE("double", Double.class,"Double");


    // 数据库源类型
    private final String sourceType;
    // 映射到的Java类类型
    private final Class<?> targetType;
    // Java类型的字符串表示
    private final String javaType;



    public String getSourceType() {
        return sourceType;
    }

    public Class<?> getTargetType() {
        return targetType;
    }

    public String getJavaType() {
        return javaType;
    }



    public static Class convertType(String type){
        for (DataType dataType : DataType.values()) {
            if (dataType.sourceType.equalsIgnoreCase(type)){
                return dataType.targetType;
            }
        }
        return String.class;
    }

    public static DataType findBySqlType(String sqlType){
        for (DataType dataType : DataType.values()) {
            if (dataType.getSourceType().equalsIgnoreCase(sqlType)){
                return dataType;
            }
        }
        return VARCHAR;
    }

    public static String convertTypeString(String type){
        for (DataType dataType : DataType.values()) {
            if (dataType.sourceType.equalsIgnoreCase(type)){
                return dataType.javaType;
            }
        }
        return "String";
    }

    DataType(String sourceType, Class<?> targetType, String javaType) {
        this.sourceType = sourceType;
        this.targetType = targetType;
        this.javaType = javaType;
    }

}
