package com.muyu.server.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.domain.*;
import com.muyu.server.service.DataLinkService;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 *@Author：yang
 *@Package：com.muyu.server.controller
 *@Project：cloud-property
 *@name：DataLinkController
 *@Date：2024/9/7 20:12
 */
@RestController
@AllArgsConstructor
@RequestMapping("/dataLink")
@Log4j2
public class DataLinkController {

    private final DataLinkService dataLinkService;

    /**
     * 查询表数据count
     * @param dataName 数据库名称
     * @param sql sql语句
     * @return 反护试图
     */
    @PostMapping("/findCount")
    public Result<Long> findCount(@RequestParam("dataName") String dataName,@RequestParam("sql") String sql){
        Long longs = dataLinkService.findCount(dataName,sql);
        return Result.success(longs);
    }

    /**
     * 查询表数据count
     * @param databaseName 数据库名称
     * @param tableName sql语句
     * @param listList sql语句
     * @return 反护试图
     */
    @PostMapping("/addProduct")
    public Result addProduct(@RequestParam("databaseName") String databaseName,
                             @RequestParam("tableName") String tableName,
                             @RequestBody DataValue[][] listList){
        long begin = System.currentTimeMillis();

        int i = dataLinkService.addProduct(databaseName,tableName,listList);


        long end = System.currentTimeMillis();

        long allTime = end - begin;
        log.info("添加到产品数据库耗时:"+allTime);

        return Result.success(i);
    }

    /**
     * 查询表数据dashuju表
     * @param databaseName 数据库名称
     * @param sql sql语句
     * @return 反护试图
     */
    @PostMapping("/findSelectDataList")
    public Result<DataValue[][]> findSelectDataList(
            @RequestParam(value = "databaseName") String databaseName,
            @RequestParam(value = "sql") String sql,
            @RequestParam(value = "tableName") String tableName,
            @RequestParam(value = "one") Long one,
            @RequestParam(value = "two") Integer two){
        DataValue[][] longs = dataLinkService.findSelectDataList(databaseName,sql,tableName,one,two);
        return Result.success(longs);
    }

    /**
     * 查询表数据下所有数据表名称
     * @return 反护试图
     */
    @PostMapping("/findSelectDataShow")
    public Result<List<DataName>> findSelectDataShow(){
        List<DataName> longs = dataLinkService.findSelectDataShow();
        return Result.success(longs);
    }

    /**
     * 查询数据下所有表名
     * @param id 数据库表名ID
     * @return 反护试图
     */
    @PostMapping("/findSelectTableShow")
    public Result<List<TableNames>> findSelectTableShow(
            @RequestParam("id") Integer id){
        List<TableNames> longs = dataLinkService.findSelectTableShow(id);
        return Result.success(longs);
    }

    /**
     * 查询表数据dashuju表
     * @param dataTableName 查看表下的所有数据
     * @return 反护试图
     */
    @PostMapping("/findSelectTableFieShow")
    public Result<List<TableFie>> findSelectTableFieShow(
            @RequestParam(value = "dataTableName") String dataTableName){
        List<TableFie> longs = dataLinkService.findSelectTableFieShow(dataTableName);
        return Result.success(longs);
    }

//    /**
//     * 查询表数据dashuju表
//     * @param dataTableName 查看表下的所有数据
//     * @return 反护试图
//     */
//    @PostMapping("/AddInsertTableShow")
//    public Result<List<TableNames>> AddInsertTableShow(
//            @RequestParam("dataTableName") String dataTableName){
//        List<TableNames> longs = dataLinkService.AddInsertTableShow(dataTableName);
//        return Result.success(longs);
//    }

}
