package com.muyu.server.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.domain.DataName;
import com.muyu.server.service.DataNameService;
import com.muyu.server.service.DataSourceService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.AllArgsConstructor;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @Author：yang
 * @Package：com.muyu.server.controller  抽取数据库名称
 * @Project：cloud-property
 * @name：DataNameController
 * @Date：2024/8/27 21:31
 */
@RestController
@RequestMapping("/dataName")
@Tag(name = "抽取数据名",description = "从数据源抽取数据名")
@AllArgsConstructor
public class DataNameController {

    private final DataNameService dataNameService;
    /**
     * 获取数据库名
     * @return 返回试图
     */
    @PostMapping("/list")
    @Operation(summary = "获取数据库名",description = "从数据库中抽取数据库名称")
    public Result<List<DataName>> extractDataName(){
        List<DataName> tableName = dataNameService.extractDataName();
        if (StringUtils.isEmpty(tableName)){
            return Result.success(null,"未查询到结构，请检查数据来源");
        }
        return Result.success(tableName,"您的数据已成功获取");
    }
}
