package com.muyu.server.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.domain.TableNames;
import com.muyu.server.service.TableRunNameService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.AllArgsConstructor;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @Author：yang
 * @Package：com.muyu.server.controller  获取数据库表名
 * @Project：cloud-property
 * @name：TableRunNameController
 * @Date：2024/8/27 20:52
 */
@RestController
@RequestMapping("/tableRunName")
@Tag(name = "抽取数据表名",description = "从数据源抽取数据表名")
@AllArgsConstructor
public class TableRunNameController {

    private final TableRunNameService tableRunNameService;
    /**
     * 获取数据库表名
     * @return 返回试图
     */
    @PostMapping("/list")
    @Operation(summary = "获取数据库表名",description = "从数据库中抽取数据库表名称")
    public Result<List<TableNames>> extractTableName(String name){
        List<TableNames> tableFIES = tableRunNameService.extractTableName(name);
        if (StringUtils.isEmpty(tableFIES)){
            return Result.success(tableFIES,"未查询到结构，请检查数据来源");
        }
        return Result.success(tableFIES,"您的数据已成功获取");
    }
}
