package com.muyu.server.mysql;



import com.muyu.common.core.utils.SpringUtils;
import com.muyu.common.domain.Connect;
import com.muyu.common.domain.DataValue;
import com.muyu.common.domain.enums.DataType;
import com.muyu.common.domain.req.ConnectReq;
import com.muyu.common.domain.data.base.BaseDataAbsSource;
import com.muyu.server.service.ConnectService;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Component;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;


/**
 * @Author：
 * @Package：com.muyu.etl.property.mysql
 * @Project：cloud-etl-property
 * @name：MysqlDataSource
 * @Date：2024/8/29 16:04
 *
 * mysql数据源
 */

@Component
@Log4j2
public class MySqlDataSource extends BaseDataAbsSource
{
    ConnectService connectService = SpringUtils.getBean(ConnectService.class);

    @Override
    public DataValue getDateValue() {
        MySqlQuery query = getQuery();
        query.getDataSourceId();
        String sql = query.getSql();
        Map<String, Object> queryParams = query.getParams();
        Connection connection=null;
        List<DataValue> dataValueList = new ArrayList<>();
        try {
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            ResultSet resultSet = preparedStatement.getResultSet();

            ResultSetMetaData metaData = resultSet.getMetaData();
            int columnCount = metaData.getColumnCount();
            while (resultSet.next()){
                for (int i = 0; i < columnCount; i++) {
                    String columnTypeName = metaData.getColumnTypeName(i);
                    // 构建DataValue对象
                    DataValue build = DataValue.builder()
                            .key(resultSet.getCursorName())// 使用游标的名称作为键
                            .label("")// 标签为空
                            .value(resultSet.getObject(resultSet.getCursorName(), DataType.convertType(columnTypeName)))
//                        .type("" +"String");// 数据类型为String
                            .build();
                    dataValueList.add(build);
                }
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }

        return null;
    }

    @Override
    public DataValue[] getRow() {
        return new DataValue[0];
    }

    @Override
    public DataValue[][] fetRows() {
        return new DataValue[0][];
    }

    public DataValue[][] getSqlDataSource() {

        MySqlQuery query = getQuery();
        String dataName = query.getDatabaseName();
        String dataTableName = query.getTableName();
        Integer one = Math.toIntExact(query.getOne());
        Integer two = query.getTwo();
        String sql = query.getSql();
        ConcurrentHashMap<Integer, DataValue> map = new ConcurrentHashMap<>();

        Connect connect = connectService.selectData(dataName);
        ConnectReq connectReq = new ConnectReq();
        HikariConfig hikariConfig = new HikariConfig();
        hikariConfig.setPoolName("HikariCP 连接池");
        hikariConfig.setDriverClassName(connectReq.getDRIVER());
        // 修正这里
        hikariConfig.setJdbcUrl(connectReq.getUrl(connect));
        hikariConfig.setUsername(connectReq.getUSER());
        hikariConfig.setPassword(connectReq.getPWD());
        hikariConfig.setMinimumIdle(2);
        hikariConfig.setMaximumPoolSize(10);

        DataValue[][] dataValues = new DataValue[one][two];

        HikariDataSource hikariDataSource = new HikariDataSource(hikariConfig);
        try {
            Connection conn = hikariDataSource.getConnection();
            PreparedStatement preparedStatement = conn.prepareStatement(sql);
            ResultSet resultSet = preparedStatement.executeQuery();
            ResultSetMetaData metaData = resultSet.getMetaData();
            int columnCount = metaData.getColumnCount();
            int c = 0;
            while (resultSet.next()){
                for (int i = 1; i <= columnCount; i++) {
                    if (resultSet.isFirst()){
                        String columnTypeName = metaData.getColumnTypeName(i);
                        DatabaseMetaData metaDataColumns = conn.getMetaData();
                        ResultSet columns = metaDataColumns.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));
                        String remarks =null;

                        while (columns.next()){
                            remarks = columns.getString("REMARKS");

                            log.info("字段备注:{}", remarks);
                        }
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(remarks)
                                .value(resultSet.getObject(i, DataType.convertType(columnTypeName)))
                                .type(DataType.findBySqlType(columnTypeName))
                                .build();
                        map.put(i,build);
                        dataValues[c][i-1]=build;
                    }else {
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(map.get(i).getLabel())
                                .value(resultSet.getObject(i, map.get(i).getType().getTargetType()))
                                .type(map.get(i).getType())
                                .build();
                        dataValues[c][i-1]=build;
                    }
                }
                c++;
            }

        } catch (SQLException e) {
            throw new RuntimeException(e);
        }finally {
            close(hikariDataSource); // 关闭数据源
        }
        log.info("数据:{}", dataValues);
        return dataValues;
    }
    private static void close(HikariDataSource dataSource) {
        if (dataSource != null) {
            dataSource.close();
        }
    }
}
