package com.muyu.server.service.impl;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.pool.DruidPooledConnection;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.muyu.common.domain.*;
import com.muyu.common.domain.data.base.BaseQueryHandler;
import com.muyu.common.domain.enums.DataType;
import com.muyu.common.domain.req.ConnectReq;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.server.mysql.MySqlDataSource;
import com.muyu.server.mysql.MySqlQuery;
import com.muyu.server.service.*;
import com.muyu.server.util.JdbcHelper;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.Date;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 *@Author：yang
 *@Package：com.muyu.server.service.impl
 *@Project：cloud-property
 *@name：DataLinkServiceImpl
 *@Date：2024/9/7 20:18
 */
@Service
@Log4j2
public class DataLinkServiceImpl implements DataLinkService {
    @Autowired
    private DataNameService dataNameService;
    @Autowired
    private TableRunNameService tableRunNameService;
    @Autowired
    private DataRunNameService dataRunNameService;
    @Autowired
    private TableFieldService tableFieldService;
    @Autowired
    private ConnectService connectService;
    /**
     * 查询表数据count
     * @param dataName 数据库名称
     * @param sql sql语句
     * @return 反回试图
     */
    @Override
    public Long findCount(String dataName, String sql) {
        try {
            DruidDataSource connRs = JdbcHelper.getConnRs(dataName);
            assert connRs != null;
            DruidPooledConnection connection = connRs.getConnection();
            PreparedStatement ps = connection.prepareStatement(sql);
            ResultSet rs = ps.executeQuery();
            while (rs.next()) {
                return rs.getLong(1);
            }
            connection.close();
        } catch (SQLException e) {
            log.error("异常信息：{}",e.getMessage());
        }
        return null;
    }

    @Override
    public DataValue[][] findSelectDataList(String databaseName, String sql, String tableName, Long one, Integer two) {

        MySqlQuery mySqlQuery = new MySqlQuery();
        mySqlQuery.setSql(sql);
        mySqlQuery.setDatabaseName(databaseName);
        mySqlQuery.setTableName(tableName);
        mySqlQuery.setOne(one);
        mySqlQuery.setTwo(two);
        BaseQueryHandler.set(mySqlQuery);
        MySqlDataSource mySqlDataSource = new MySqlDataSource();
        DataValue[][] rows = mySqlDataSource.getSqlDataSource();
        log.info("rows：{}",rows);
        return rows;
    }

    @Override
    public List<DataName> findSelectDataShow() {

        List<DataName> dataNameList = dataNameService.list()
                .stream()
                .map(DataName::dataNameBuild)
                .toList();
        return dataNameList;
    }

    /**
     * 查询表数据dashuju表
     * @param id 查看表下的所有数据
     * @return 反护试图
     */
    @Override
    public List<TableNames> findSelectTableShow(Integer id) {

        List<TableNames> tableNamesList = tableRunNameService.list(new LambdaQueryWrapper<TableNames>()
                .eq(TableNames::getDataId, id))
                .stream()
                .map(TableNames::tableNamesBuild)
                .toList();
        return tableNamesList;
    }

    @Override
    public List<TableFie> findSelectTableFieShow(String dataTableName) {
        //在取数据数据遍历里获取表名称查询方法
        List<TableNames> tableNamesList = tableRunNameService.list(new LambdaQueryWrapper<TableNames>()
                        .eq(TableNames::getName, dataTableName))
                .stream()
                .map(TableNames::tableNamesBuild)
                .toList();
        List<TableFie> tableFieList = new ArrayList<>();
        tableNamesList.forEach(tableNames -> {
            List<TableFie> tableFieTs = this.tableStructureShow(tableNames);
            tableFieList.addAll(tableFieTs);
        });
        return tableFieList;
    }

    /**
     * 添加方法
     * @param databaseName 数据库名称
     * @param tableName sql语句
     * @param listList sql语句
     * @return
     */
    @Override
    @Log(title = "添加产品数据库", businessType = BusinessType.INSERT)
    public int addProduct(String databaseName, String tableName, DataValue[]... listList) {
        log.info("DATA:{}",listList);
        Connect connect = connectService.selectData(databaseName);

        ConnectReq connectReq = new ConnectReq();
        HikariConfig hikariConfig = new HikariConfig();
        hikariConfig.setPoolName("HikariCP 连接池");
        hikariConfig.setDriverClassName(connectReq.getDRIVER());
        // 修正这里
        hikariConfig.setJdbcUrl(connectReq.getUrl(connect));
        hikariConfig.setUsername(connectReq.getUSER());
        hikariConfig.setPassword(connectReq.getPWD());
        hikariConfig.setMinimumIdle(2);
        hikariConfig.setMaximumPoolSize(10);
        hikariConfig.setMaxLifetime(300000); // 5 minutes
        hikariConfig.setConnectionTimeout(30000);

        HikariDataSource hikariDataSource = new HikariDataSource(hikariConfig);

        ExecutorService executorService = Executors.newFixedThreadPool(6);
        AtomicInteger addCount = new AtomicInteger();

        // 分割数据为较小的批次
        List<DataValue[][]> batches = splitData(listList, 5000);

        try (Connection conn = hikariDataSource.getConnection()) {
            conn.setAutoCommit(false); // 开启事务

            for (DataValue[][] batch : batches) {
                executorService.submit(() -> {
                    try (Statement stmt = conn.createStatement()) {
                        String sql = buildBatchInsertSQL(tableName, batch);
                        stmt.executeUpdate(sql);
                        addCount.addAndGet(batch.length);
                    } catch (SQLException e) {
                        log.error("SQLException异常发生", e);
                        try {
                            conn.rollback(); // 回滚事务
                        } catch (SQLException ex) {
                            log.error("回滚事务失败", ex);
                            throw new RuntimeException(ex);
                        }
                        throw new RuntimeException(e);
                    } catch (Exception e) {
                        log.error("其他异常发生", e);
                        try {
                            conn.rollback(); // 回滚事务
                        } catch (SQLException ex) {
                            log.error("回滚事务失败", ex);
                            throw new RuntimeException(ex);
                        }
                        throw new RuntimeException(e);
                    }
                });
            }

            executorService.shutdown();
            if (!executorService.awaitTermination(1, TimeUnit.HOURS)) {
                log.warn("Executor service did not terminate within the timeout.");
                executorService.shutdownNow();
            }
            conn.commit(); // 提交事务
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            throw new RuntimeException(e);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
            close(hikariDataSource); // 关闭数据源
        }

        return addCount.get();

    }

    private static void close(HikariDataSource dataSource) {
        if (dataSource != null) {
            dataSource.close();
        }
    }

    private List<DataValue[][]> splitData(DataValue[][] listList, int batchSize) {
        List<DataValue[][]> batches = new ArrayList<>();
        int totalRows = listList.length;
        int totalBatches = (int) Math.ceil((double) totalRows / batchSize);

        for (int i = 0; i < totalBatches; i++) {
            int start = i * batchSize;
            int end = Math.min(start + batchSize, totalRows);
            DataValue[][] batch = Arrays.copyOfRange(listList, start, end);
            batches.add(batch);
        }

        return batches;
    }

    private String buildBatchInsertSQL(String tableName, DataValue[][] batch) {
        StringBuilder columns = new StringBuilder("(");
        StringBuilder values = new StringBuilder("VALUES ");

        // 构建字段名
        for (DataValue dataValue : batch[0]) {
            String key = dataValue.getKey();
            columns.append(key).append(", ");
        }
        // 删除最后一个逗号和空格
        columns.delete(columns.length() - 2, columns.length());

        // 构建值部分
        for (DataValue[] dataValueList : batch) {
            values.append("(");
            for (DataValue dataValue : dataValueList) {
                Object value = dataValue.getValue();
                values.append(formatValue(dataValue.getType(), value)).append(", ");
            }
            values.delete(values.length() - 2, values.length());
            values.append("), ");
        }
        // 删除最后一个逗号
        values.delete(values.length() - 2, values.length());

        // 完成 SQL 插入语句
        String sql = "INSERT INTO " + tableName + " " + columns.toString() + ") " + values.toString();
        return sql;
    }

    private String formatValue(DataType type, Object value) {
        if (value == null) {
            // 根据业务需求处理 null 值
            return "NULL"; // 或者其他默认值
        }

        if (type == DataType.VARCHAR || type == DataType.TEXT) {
            return "'" + value.toString().replace("'", "''") + "'";
        } else if (type == DataType.BIGINT) {
            return value.toString();
        } else if (type == DataType.INT) {
            return value.toString();
        } else if (type == DataType.DECIMAL) {
            return value.toString();
        } else if (type == DataType.DATETIME) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            return "'" + sdf.format((Date) value) + "'";
        } else if (type == DataType.DOUBLE) {
            return value.toString();
        } else {
            return "'" + value.toString() + "'";
        }
    }


    public List<TableFie> tableStructureShow(TableNames tableNames) {

        List<TableFie> tableFieList = null;
        if (StringUtils.isNotBlank(String.valueOf(tableNames.getId()))) {
            tableFieList = tableFieldService.list(new LambdaQueryWrapper<TableFie>()
                            .eq(TableFie::getTableId, tableNames.getId()))
                    .stream()
                    .map(TableFie::tableFieBuild).toList();
            log.info("字段类型名称集合：{}", tableFieList);
            DataName dataName = dataNameService.getById(tableNames.getDataId());

            List<DataValue> tableValue = this.findTableValue(String.valueOf(dataName.getId()), tableNames.getName());

            tableFieList.forEach(tableFy -> {
                //查询字段映射信息
                String str = this.tableFySelect(dataName.getName(), tableNames.getName(), tableFy.getField());
                tableFy.setDataValue(str);

                log.info("字段数据对象:{}", tableFy);

                for (DataValue dataValue : tableValue) {
                    if (tableFy.getField().equals(dataValue.getKey())) {
                        tableFy.setDataMapping(dataValue.getType().getJavaType());
                    }
                }
            });
            log.info("字段数据信息:{}", tableFieList);
        }
        return tableFieList;
    }

    /**
     * 数据类型转换
     * @para 数据类型转换ID
     * @return 返回结果
     */
    public List<DataValue> findTableValue( String dataId,String tableName) {
        DataName byId = dataNameService.getById(dataId);
        MySqlQuery mySqlQuery = new MySqlQuery();
        // 设置数据源ID
        mySqlQuery.setDataSourceId(String.valueOf(dataId));

        DruidDataSource connRs1 = JdbcHelper.getConnRs(byId.getName());
        DruidPooledConnection conn = null;
        try {
            conn = connRs1.getConnection();

        } catch (SQLException e) {
            throw new RuntimeException(e);
        }

        List<DataValue> list = new ArrayList<>();

        try {
            PreparedStatement preparedStatement = conn.prepareStatement("select * FROM " + tableName + " LIMIT 5");

            this.getDataResult(conn, list, preparedStatement);
            JdbcHelper.close(conn,null,null);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return list;
    }

    /**
     * 查询值
     *
     * @param dataDame
     * @param tableName
     * @param field
     * @return
     */
    private String tableFySelect(String dataDame, String tableName, String field) {

        // 从连接池获取数据库连接
        DruidDataSource conn = JdbcHelper.getConnRs(dataDame);
        // 初始化一个列表，用于存储数据值对象
        try {
            DruidPooledConnection connection = conn.getConnection();
            // 准备SQL查询语句

            PreparedStatement ps = connection.prepareStatement("select " + field + " FROM " + tableName +" LIMIT 5");
            ResultSet resultSet = ps.executeQuery();
            while (resultSet.next()) {
                String fields = resultSet.getString(field);
                // 返回包含数据值的列表
                return fields;
            }

            // 释放数据库连接
        } catch (SQLException e) {
            // 如果发生SQL异常，抛出运行时异常
            throw new RuntimeException(e);
        }
        return null;
    }

    /**
     * 授权后资产展示
     * @param conn
     * @param list
     * @param preparedStatement
     * @throws SQLException
     */
    private void getDataResult(DruidPooledConnection conn, List<DataValue> list, PreparedStatement preparedStatement) {
        try {
            ResultSet resultSet = preparedStatement.executeQuery();
            ResultSetMetaData metaData = resultSet.getMetaData();
            int columnCount = metaData.getColumnCount();

            HashMap<Integer, DataValue> map = new HashMap<>();
            while (resultSet.next()){
                for (int i = 1; i <= columnCount; i++) {

                    if (resultSet.isFirst()){
                        String columnTypeName = metaData.getColumnTypeName(i);

                        DatabaseMetaData metaData1 = conn.getMetaData();
                        ResultSet columns = metaData1.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));
                        String remarks =null;
                        while (columns.next()){
                            remarks = columns.getString("REMARKS");
                            log.info("字段备注:{}", remarks);
                        }
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(remarks)
                                .value(resultSet.getObject(i, DataType.convertType(columnTypeName)))
                                .type(DataType.findBySqlType(columnTypeName))
                                .build();
                        list.add(build);
                        map.put(i, build);
                    }else {
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(map.get(i).getLabel())
                                .value(resultSet.getObject(i, map.get(i).getType().getTargetType()))
                                .type(map.get(i).getType())
                                .build();
                        list.add(build);
                    }

                }
            }
            JdbcHelper.close(null,preparedStatement,null);
        } catch (Exception e) {
            log.error(e.getMessage(),e);
        }
    }
}
